from typing import Any, Optional

from sycamore.query.operators.logical_operator import LogicalOperator


class BasicFilter(LogicalOperator):
    """Basic filters for data when field already exists. Used in cases where LLM filter is not
    necessary.

    Returns a database.
    """

    range_filter: bool = False
    """If true, it will use an inclusive range filter (in which case you need to specify *start*
    and/or *end*). The range filter requires a specific field value to fall within a range of values.
    This is mainly used for date ranges, e.g. *start*=2022/10/01, *end*=2022/10/20, *field*=date
    requires the date to be between October 1 and October 20 (inclusive) in 2022. For range filters,
    you are not required to specify both *start* and *end* if unnecessary. If range_filter is false, a
    match filter will be used (in which case you need to specify *query*). The match filter requires a
    specific field to match a fixed value (the *query*), e.g. match 2 in "passenger_count". For strings,
    a match filter performs substring matching. For example, e.g. query="sub" would match with the value
    "submarine".
    """

    query: Optional[Any] = None
    """The value to search for when using a match filter."""

    start: Optional[Any] = None
    """The start value for the range filter."""

    end: Optional[Any] = None
    """The end value for the range filter."""

    field: str
    """The name of the database field to filter based on."""

    date: bool = False
    """Specifies if the range filter is being performed on a date."""
